Package: PopED
Type: Package
Title: Population (and Individual) Optimal Experimental Design
Version: 0.7.0
Depends: R (>= 2.14)
Imports: ggplot2, MASS, mvtnorm, dplyr (>= 0.7.0), codetools, stats,
        utils, magrittr, boot, purrr, stringr, tibble, gtools
Suggests: testthat, Hmisc, nlme, GA, deSolve, Rcpp, shiny,
        rhandsontable, knitr, rmarkdown, gridExtra, covr, devtools,
        mrgsolve
Authors@R: c(
    person("Andrew C.","Hooker", email="andrew.hooker@farmaci.uu.se", 
      role=c("aut","cre","trl","cph"),
      comment=c(ORCID = "0000-0002-2676-5912")
    ),
    person("Marco","Foracchia", role="aut", comment="O-Matrix version"),
    person("Eric","Stroemberg", role="ctb", comment="MATLAB version"),
    person("Martin","Fink", role="ctb", comment="Streamlining code, added functionality, vignettes"),
    person("Giulia","Lestini", role="ctb", comment="Streamlining code, added functionality, vignettes"),
    person("Sebastian","Ueckert", role="aut", comment=c("MATLAB version",ORCID="0000-0002-3712-0255")),
    person("Joakim", "Nyberg", role="aut", comment="MATLAB version")
    )
Description: Optimal experimental designs for both population and individual
    studies based on nonlinear mixed-effect models. Often this is based on a
    computation of the Fisher Information Matrix. This package was developed
    for pharmacometric problems, and examples and predefined models are available
    for these types of systems. The methods are described in Nyberg et al. 
    (2012) <doi:10.1016/j.cmpb.2012.05.005>, and Foracchia et al. (2004) 
    <doi:10.1016/S0169-2607(03)00073-7>.
License: LGPL (>= 3)
ByteCompile: true
URL: https://andrewhooker.github.io/PopED/,
        https://github.com/andrewhooker/PopED
BugReports: https://github.com/andrewhooker/PopED/issues
Copyright: 2014-2021 Andrew C. Hooker
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/Needs/website: mrgsolve, kableExtra, PKPDsim, rxode2
NeedsCompilation: no
Packaged: 2024-10-07 18:55:02 UTC; ancho179
Author: Andrew C. Hooker [aut, cre, trl, cph]
    (<https://orcid.org/0000-0002-2676-5912>),
  Marco Foracchia [aut] (O-Matrix version),
  Eric Stroemberg [ctb] (MATLAB version),
  Martin Fink [ctb] (Streamlining code, added functionality, vignettes),
  Giulia Lestini [ctb] (Streamlining code, added functionality,
    vignettes),
  Sebastian Ueckert [aut] (MATLAB version,
    <https://orcid.org/0000-0002-3712-0255>),
  Joakim Nyberg [aut] (MATLAB version)
Maintainer: Andrew C. Hooker <andrew.hooker@farmaci.uu.se>
Repository: CRAN
Date/Publication: 2024-10-07 19:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 14:14:22 UTC; unix
