# Generated by roxygen2: do not edit by hand

S3method(summary,poped_optim)
export(Doptim)
export(Dtrace)
export(LEDoptim)
export(LinMatrixL)
export(LinMatrixLH)
export(LinMatrixL_occ)
export(RS_opt)
export(a_line_search)
export(bfgsb_min)
export(blockexp)
export(blockfinal)
export(blockheader)
export(blockopt)
export(build_sfg)
export(calc_autofocus)
export(calc_ofv_and_fim)
export(calc_ofv_and_grad)
export(cell)
export(convert_variables)
export(create.poped.database)
export(create_design)
export(create_design_space)
export(design_summary)
export(diag_matlab)
export(ed_laplace_ofv)
export(ed_mftot)
export(efficiency)
export(evaluate.e.ofv.fim)
export(evaluate.fim)
export(evaluate_design)
export(evaluate_fim_map)
export(evaluate_power)
export(feps.add)
export(feps.add.prop)
export(feps.prop)
export(feval)
export(ff.PK.1.comp.oral.md.CL)
export(ff.PK.1.comp.oral.md.KE)
export(ff.PK.1.comp.oral.sd.CL)
export(ff.PK.1.comp.oral.sd.KE)
export(ff.PKPD.1.comp.oral.md.CL.imax)
export(ff.PKPD.1.comp.sd.CL.emax)
export(fileparts)
export(getTruncatedNormal)
export(get_all_params)
export(get_rse)
export(get_unfixed_params)
export(getfulld)
export(gradf_eps)
export(inv)
export(isempty)
export(mc_mean)
export(median_hilow_poped)
export(mfea)
export(mftot)
export(model_prediction)
export(ofv_criterion)
export(ofv_fim)
export(ones)
export(optim_ARS)
export(optim_LS)
export(optimize_groupsize)
export(optimize_n_eff)
export(optimize_n_rse)
export(pargen)
export(plot_efficiency_of_windows)
export(plot_model_prediction)
export(poped.choose)
export(poped_gui)
export(poped_optim)
export(poped_optimize)
export(rand)
export(randn)
export(shrinkage)
export(size)
export(start_parallel)
export(test_mat_size)
export(tic)
export(toc)
export(zeros)
import(ggplot2)
importFrom(codetools,findGlobals)
importFrom(magrittr,"%>%")
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,end)
importFrom(stats,optim)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(utils,capture.output)
importFrom(utils,find)
importFrom(utils,packageVersion)
importFrom(utils,stack)
